#import "ixCover.h"


int SDCompareIntegers (id obj1, id obj2, void *context)
{
    // treat NSData as containing integers
    const void *data1 = [obj1 bytes];
    const void *data2 = [obj2 bytes];
    unsigned int num1 = *(unsigned int *)data1;
    unsigned int num2 = *(unsigned int *)data2;
    int result;
    if (num1==num2) result = NSOrderedSame;
    else if (num1<num2) result =  NSOrderedAscending;
    else /*if (num1>num2)*/ result =  NSOrderedDescending;
    return result;

}

int SDCompareCombo (id obj1, id obj2, void *context)
{
    const void *data1 = [obj1 bytes];
    const void *data2 = [obj2 bytes];
    unsigned int num1 = *(unsigned int *)data1;
    unsigned int num2 = *(unsigned int *)data2;
    NSString *s1 = [NSString stringWithCString:(data1+sizeof(unsigned int)) length:[obj1 length]- sizeof(unsigned int)];
    NSString *s2 = [NSString stringWithCString:(data2+sizeof(unsigned int)) length:[obj2 length]- sizeof(unsigned int)];
    int result = [s1 caseInsensitiveCompare:s2];
    if (result == NSOrderedSame) {
        if (num1==num2) result = NSOrderedSame;
        else if (num1<num2) result =  NSOrderedAscending;
        else /* if (num1>num2) */ result = NSOrderedDescending;
    }
    return result;
}

int SDCompareStrings(id data1, id data2, void *context)
{
    // assumes the objects are NSStrings
    return [(NSString *)data1 compare:(NSString *)data2];
}



int SDCompareMonocaseStrings(id data1, id data2, void *context)
{
    // assumes the objects are NSStrings
    return [data1 caseInsensitiveCompare:data2];
}

int IXCompareMonocaseStrings(const void *data1, unsigned short length1, const void *data2, unsigned short length2, const void *context)
{
    NSString *s1 = [NSString stringWithCString:data1 length:length1];
    NSString *s2 = [NSString stringWithCString:data2 length:length2];
    return SDCompareMonocaseStrings(s1,s2,(void *)context);
}

int SDCompareMonocaseStringData (id obj1, id obj2, void *context)
{
    // treat the NSData * as containing strings
    NSString *s1 = [NSString stringWithCString:[obj1 bytes] length:[obj1 length]];
    NSString *s2 = [NSString stringWithCString:[obj2 bytes] length:[obj2 length]];
    return SDCompareMonocaseStrings(s1,s2,context);
}

int SDCompareStringData (id obj1, id obj2, void *context)
{
    // treat the NSData * as containing strings
    NSString *s1 = [NSString stringWithCString:[obj1 bytes] length:[obj1 length]];
    NSString *s2 = [NSString stringWithCString:[obj2 bytes] length:[obj2 length]];
    return SDCompareStrings(s1,s2,context);
}

// ACK need to worry about endian-ness
int SDCompareUnsignedLongs (id obj1, id obj2, void *context)
{
    // treat NSData as containing unsigned longs
    const void *data1 = [obj1 bytes];
    const void *data2 = [obj2 bytes];
    unsigned long num1 = *(unsigned long *)data1;
    unsigned long num2 = *(unsigned long *)data2;
    int result;
    if (num1==num2) result = NSOrderedSame;
    else if (num1<num2) result =  NSOrderedAscending;
    else /* if (num1>num2)*/ result =  NSOrderedDescending;
    return result;
}

int IXCompareUnsignedLongs(const void *data1, unsigned short length1, const void *data2, unsigned short length2, const void *context)
{
    unsigned long num1 = *(unsigned long *)data1;
    unsigned long num2 = *(unsigned long *)data2;
    int result;
    if (num1==num2) result = NSOrderedSame;
    else if (num1<num2) result =  NSOrderedAscending;
    else /*if (num1>num2)*/ result =  NSOrderedDescending;
    return result;
}

int IXCompareDouble(const void *data1, unsigned short length1, const void *data2, unsigned short length2, const void *context)
{
    double num1 = *(double *)data1;
    double num2 = *(double *)data2;
    int result;
    if (num1==num2) result = NSOrderedSame;
    else if (num1<num2) result =  NSOrderedAscending;
    else /* if (num1>num2) */ result =  NSOrderedDescending;
    return result;
}

int IXCompareLong(const void *data1, unsigned short length1, const void *data2, unsigned short length2, const void *context)
{
    long num1 = *(long *)data1;
    long num2 = *(long *)data2;
    int result;
    if (num1==num2) result = NSOrderedSame;
    else if (num1<num2) result =  NSOrderedAscending;
    else /* if (num1>num2) */ result =  NSOrderedDescending;
    return result;
}

int IXCompareFloat(const void *data1, unsigned short length1, const void *data2, unsigned short length2, const void *context)
{
    float num1 = *(float *)data1;
    float num2 = *(float *)data2;
    int result;
    if (num1==num2) result = NSOrderedSame;
    else if (num1<num2) result =  NSOrderedAscending;
    else /* if (num1>num2) */ result =  NSOrderedDescending;
    return result;
}
